/** 
 @file player.js
 
 @brief This file contain implementation of methods required to store data from UI.

 @author: mobile2DS

 @date: 12.1.2014
*/

var lWarn = {logln:function (o){}}; 
var lInfo = {logln:function (o){}};

var LOGFILE_NAME = "sonos-controller.log";
lWarn = new Logger(LOGFILE_NAME);
lInfo = new Logger(LOGFILE_NAME);


/**
 @brief unregisters trigger for Activity. We have special scenario for security event when we must also unregister undo triggger
 
 @param oNode [in] node of Activity 
*/
function unRegisterActivityTrigger(oNode) {
  var sPath = oNode.getPath();
  
  lInfo.logln("unregisterTrigger trigger started -> "+sPath);
  unregisterTrigger(sPath);
  
  // for security events we unregister also undo trigger
  var pUndoTrigger = oNode.getChild('undoCallScene');
  if (pUndoTrigger != null) {
    unregisterTrigger(sPath+"/undoCallScene");
  }
  lInfo.logln("Registering trigger finished");
}                   

/**
 @brief registers trigger for Activity. We have special scenario for security event when we must also register undo triggger
 
 @param oNode [in] node of Activity 
 @param sTriggerType [in] Name of trigger type
  
*/
function registerActivityTrigger(oNode, sTriggerType) {
  var sPath = oNode.getPath();
  
  lInfo.logln("Registering trigger started -> "+sPath);
  registerTrigger(sPath, sTriggerType, {});
  
  // for security evente we register also undo trigger
  var pUndoTrigger = oNode.getChild('undoCallScene');
  if (pUndoTrigger != null) {
    registerTrigger(sPath+"/undoCallScene" , "sonos_execute_undo", {});
  }
  lInfo.logln("Registering trigger finished");
}                   

/**
 * Store a boolean property parameter.
 * convert value if possible. In case In case conversion is not possible 
 * the value is discarded.
 * @param path [in] property path
 * @param value [in] value to store should be a boolean
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreBoolParameter(path, value) {
    try 
    {
        lInfo.logln('store boolean at |' + path + "| value: |" + value + "|" );
        var successCode = false;
        var boolValue = false;
        if (typeof(value) === "boolean") {
            Property.setProperty(path, value);
            Property.setFlag(path, 'ARCHIVE', true);       
            successCode = true;
        } else {
            if (typeof(value) === "string") {
                if (value === "true") {
                    boolValue = true;
                }
                Property.setProperty(path, boolValue);
                Property.setFlag(path, 'ARCHIVE', true);       
                successCode = true;
            } else {
                lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
            }
        }
        return successCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreBoolParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Store a string property parameter.
 * check weather the provided value is a string. In case it's a number it is converted to string, 
 * in any other cases it is discarded
 * @param path [in] property path
 * @param value [in] value to store should be string or at least a number
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreStringParameter(path, value) {
    try 
    {
        lInfo.logln('store string at |' + path + "| value: |" + value + "|" );
        var sucessCode = false;
        var stringValue = (typeof(value) === "number") ? String(value) : value;
        if (typeof(stringValue) === "string") {
            Property.setProperty(path, stringValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else {
            lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
        }
        return sucessCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreStringParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Store a integer property parameter.
 * converts provided value whenever possible to an integer. Discard the the value if this is not possible.
 * @param path [in] property path
 * @param value [in] value to store should be integer
 * @returns {Boolean} returns false if value could not be converted to integer
 */
function checkAndStoreIntParameter(path, value) {
    try 
    {
        lInfo.logln('store integer at |' + path + "| value: |" + value + "|" );
        var sucessCode = false;
        var intValue = parseInt("" + value);
        if (!isNaN(intValue)) {
            Property.setProperty(path, intValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else {
            lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
        }
        return sucessCode;
    }
    catch (e)
    {
        lWarn.logln("Exception beim checkAndStoreIntParameter " + e + " Path " + path  + " value " + value);
        return false;
    }        
}

/**
 * Stores trigger component data to property tree
 * converts provided value whenever possible to an integer. Discard the the value if this is not possible.
 * @param idValue [in] id of activity
 * @param oParameterObject [in] trigger component data
 * @returns {Boolean} returns true if succeed
 */
function putTriggerParameter(idValue,oParameterObject) {
 var iIndex;
 var triggerIndexCounter = 0;   
     
    for (iIndex=0;iIndex<oParameterObject.triggers.length;iIndex++)
    {
      lInfo.logln(JSON.stringify(oParameterObject.triggers[iIndex]));
      if (!oParameterObject.triggers[iIndex].bSkip) {
        var sSubpath= 'sonos-activities/'+idValue + '/triggers/' + (triggerIndexCounter + 1) + "/";
        switch(oParameterObject.triggers[iIndex].type) {       
            case 'zone-scene':
            case 'bus-zone-scene':
                // normal zone scene trigger
                checkAndStoreStringParameter(sSubpath + "type", 'bus-zone-scene');
                checkAndStoreIntParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone);
                checkAndStoreIntParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group);
                checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                if (oParameterObject.triggers[iIndex].forced!=null)
                  checkAndStoreBoolParameter(sSubpath + "forced" , oParameterObject.triggers[iIndex].forced);
                storeSubParameterFull(sSubpath + "dsid" , -1);

              if (predefinedUnduEvents.indexOf(oParameterObject.triggers[iIndex].scene) != -1) {
                // we create additional trigger for UNDO scene
                sSubpath= 'sonos-activities/'+idValue + '/undoCallScene/triggers/' + (triggerIndexCounter + 1) + "/";
                checkAndStoreStringParameter(sSubpath + "type" , 'undo-zone-scene');
                checkAndStoreIntParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone);
                checkAndStoreIntParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group);
                checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                storeSubParameterFull(sSubpath + "dsid" , -1);
              }
            break;
            case 'undo-zone-scene':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreIntParameter(sSubpath + "zone" , oParameterObject.triggers[iIndex].zone);
             checkAndStoreIntParameter(sSubpath + "group" , oParameterObject.triggers[iIndex].group);
             checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                storeSubParameterFull(sSubpath + "dsid" , -1);
                break;
            case 'device-msg':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
             checkAndStoreIntParameter(sSubpath + "msg" , oParameterObject.triggers[iIndex].msg);
                if (oParameterObject.triggers[iIndex].buttonIndex!=null)
                 checkAndStoreIntParameter(sSubpath + "buttonIndex" , oParameterObject.triggers[iIndex].buttonIndex);
                break;
            case 'device-binary-input':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
             checkAndStoreIntParameter(sSubpath + "index" , oParameterObject.triggers[iIndex].index);
             checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
                break;
            case 'device-scene':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
             checkAndStoreIntParameter(sSubpath + "scene" , oParameterObject.triggers[iIndex].scene);
                break;
            case 'custom-event':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreIntParameter(sSubpath + "event" , oParameterObject.triggers[iIndex].event);
                break;
            case 'device-sensor':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "dsid" , oParameterObject.triggers[iIndex].dsid);
             checkAndStoreStringParameter(sSubpath + "eventid" , oParameterObject.triggers[iIndex].eventid);
                break;
            case 'state-change':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name);
             checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
             
             if (oParameterObject.triggers[iIndex].name == "fire") {
                sSubpath= 'sonos-activities/'+idValue + '/undoCallScene/triggers/' + (triggerIndexCounter + 1) + "/";
                checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
                checkAndStoreStringParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name);
                checkAndStoreStringParameter(sSubpath + "state" , "inactive");
             }
                break;
            case 'addon-state-change':
             checkAndStoreStringParameter(sSubpath + "type" , oParameterObject.triggers[iIndex].type);
             checkAndStoreStringParameter(sSubpath + "addon-id" , oParameterObject.triggers[iIndex].addonId);
             checkAndStoreStringParameter(sSubpath + "name" , oParameterObject.triggers[iIndex].name);
             checkAndStoreStringParameter(sSubpath + "state" , oParameterObject.triggers[iIndex].state);
                break;
        }
        triggerIndexCounter++;
      }
    }
}

/** 
 @brief Creates new ID required for storing data to PropertyTree
 
 @param sNode [in] node from whitch to get new ID
  
 @return New ID value
*/
function getNewIdValue(sNode) {
    var idValue = 0;
    var baseNode = Property.getNode(sNode);
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    if (oArrayNode != null) {
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          if (oChildNode.getChild('id') != null) {
              if (oChildNode.getChild('id').getValue() != null)
                  if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                      idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
          }
      }
    }
    return idValue;
}

/** 
 @brief Searching for player in PropertyTree
 
 @param sMasterPlayerID [in] main playerID
 @param sSlavePlayers [in] list of sub players separated with comma
  
 @return Id of searched player
*/
function getDefaultDeviceID(sMasterPlayerID, sSlavePlayers) {
   var idValue = -1;
  
   var arrayPlayers = new Array(); // list of all players in sonos group
   arrayPlayers.push(sMasterPlayerID);
   
   var bHasSlaves = false;
   if (sSlavePlayers.length>0) {
     var aSlaves = sSlavePlayers.split(","); 
     if (aSlaves != null && aSlaves.length > 0) {
       for (var iIndex=0;iIndex<aSlaves.length;iIndex++) {
         if (aSlaves[iIndex].trim().length > 0) {
           arrayPlayers.push(aSlaves[iIndex].trim());
           bHasSlaves = true;
         }
       }
     }
   }
   
   var baseNode = Property.getNode('sonos-players');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('deviceID') != null) {
           if (arrayPlayers.indexOf(oChildNode.getChild('deviceID').getValue())>-1) {
             if (bHasSlaves) {  
               if (oChildNode.getChild('playersListSlaves') != null && oChildNode.getChild('playersListSlaves').getValue().length>0) {
                 var aStoredSlaves = oChildNode.getChild('playersListSlaves').getValue().split(",");
                 if (aStoredSlaves != null && aStoredSlaves.length == (arrayPlayers.length-1)) {
                   bAllSubFound = true;
                   for ( var yIndex = 0; yIndex < aStoredSlaves.length; yIndex++) {
                     if (arrayPlayers.indexOf(aStoredSlaves[yIndex])<0) {
                       bAllSubFound = false;
                       break;
                     }
                   }
                   if (bAllSubFound) {
                     idValue = oChildNode.getChild('id').getValue();
                     break;
                   }
                 }
               }
             }
             else {
               if (sMasterPlayerID == oChildNode.getChild('deviceID').getValue() && oChildNode.getChild('playersListSlaves') != null && oChildNode.getChild('playersListSlaves').getValue().length==0) {
                 idValue = oChildNode.getChild('id').getValue();
                 break;
               }
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Searching for preset action in PropertyTree
 
 @param sPlayerID [in] id of player
 @param nPresetID [in] seacrhed preset ID
  
 @return Id of searched preset action, or -1 if not found
*/
function getDefaultPresetID(nRoomID, nPresetID) {
   var idValue = -1;
   var baseNode = Property.getNode('sonos-presets');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('roomID') != null) {
           if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
             if (oChildNode.getChild('presetID') != null) {
               if (oChildNode.getChild('presetID').getValue() != null && oChildNode.getChild('presetID').getValue() == nPresetID) {
                 if (oChildNode.getChild('id').getValue() != null) {
                   idValue = oChildNode.getChild('id').getValue()
                   break;
                 }
               }
             }
           }
         }
     }
   }
   return idValue;
}

/** 
 @brief Searching dSS room from list with sonos group 
 
 @param roomsList [in] list of rooms
 @param sSonosGroup [in] sonos groupID
  
 @return Id of dSS room
*/
function getStoredRoom(roomsList, sSonosGroup) {
  var nRoomID = 0;
  for ( var iIndex = 0; iIndex < roomsList.length; iIndex++) {
    var oChildNode = roomsList[iIndex];
    if (oChildNode.groupID == sSonosGroup) {
       nRoomID = oChildNode.roomID;
       break;
    }
  }
  return nRoomID;
}

/** 
 @brief Storing Sonos Playlist to PropertyTree 
 
 @param arrayList [in] list of Sonos Playlist
  
 @return 
*/
function saveSonosPlayList(arrayList) {
    try {
        // Delete old list
        var baseNode = Property.getNode('sonos-playlists');
        if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               var oChildNode = oArrayNode[iIndex];
               baseNode.removeChild(oChildNode.getChild('id').getValue());
             }
        }
     
        if (arrayList != null) {
          for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
               var sData = arrayList[iIndex].data;      
            storeSubParameter('sonos-playlists', iIndex+'/id', iIndex);
            storeSubParameter('sonos-playlists', iIndex+'/title', unescape(htmlUnescape(arrayList[iIndex].title)));
            storeSubParameter('sonos-playlists', iIndex+'/listURL', unescape(arrayList[iIndex].listCall));
            storeSubParameter('sonos-playlists', iIndex+'/playListSonosID', unescape(arrayList[iIndex].listID));
          }
        }
    } 
    catch (e) {
        lInfo.logln("Error by saving saveSonosPlayList: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
}

/** 
 @brief Storing Sonos Favorites to PropertyTree 
 
 @param arrayList [in] list of Sonos Favorites
  
 @return 
*/
function saveSonosFavorites(arrayList) {
    try {
     // Delete old list
        var baseNode = Property.getNode('sonos-favorites');
        if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               var oChildNode = oArrayNode[iIndex];
               baseNode.removeChild(oChildNode.getChild('id').getValue());
             }
        }
     
        if (arrayList != null) {
          for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
               var sData = arrayList[iIndex].data;      
            storeSubParameter('sonos-favorites', iIndex+'/id', iIndex);
            storeSubParameter('sonos-favorites', iIndex+'/title', unescape(htmlUnescape(arrayList[iIndex].title)));
            storeSubParameter('sonos-favorites', iIndex+'/description', unescape(htmlUnescape(arrayList[iIndex].description)));
            storeSubParameter('sonos-favorites', iIndex+'/listURL', unescape(arrayList[iIndex].listCall));            
            storeSubParameter('sonos-favorites', iIndex+'/playListSonosID', unescape(arrayList[iIndex].listID));
            storeSubParameter('sonos-favorites', iIndex+'/isSingleTrack', JSON.parse(arrayList[iIndex].isSingleTrack));
          }
        }
    } 
    catch (e) {
        lInfo.logln("Error by saving saveSonosFavorites: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }                       
}

/** 
 @brief Stores Sonos Player data to PropertyTree
 
 @param objPlayer [in] - Object with Sonos player data
  
 @return 
*/ 
function storePlayer(objPlayer, arrayAssignedRooms) {
    try {
        var playerPropertyID = getDefaultDeviceID(objPlayer.playerID, objPlayer.playersInGroup);
        var bNewDevice = false; 

        if (playerPropertyID < 0) {
          playerPropertyID = getNewIdValue('sonos-players');
          bNewDevice = true;
          
          var nRoomID = 0;
          if (arrayAssignedRooms != null && arrayAssignedRooms.length>0){
            for (iIndex=0;iIndex<arrayAssignedRooms.length;iIndex++) {
              if (arrayAssignedRooms[iIndex].deviceID == unescape(objPlayer.playerID)) {
                nRoomID = arrayAssignedRooms[iIndex].roomID;
                break;
              }
            }
          }
          
          storeSubParameter('sonos-players', playerPropertyID+'/roomID', nRoomID);
        }
                                                                          
        // storing data to propertyTree
        storeSubParameter('sonos-players', playerPropertyID+'/id', playerPropertyID);
        storeSubParameter('sonos-players', playerPropertyID+'/ipAddress', unescape(objPlayer.deviceIP));
        storeSubParameter('sonos-players', playerPropertyID+'/sonosRoom', unescape(objPlayer.sonosRoom));
        storeSubParameter('sonos-players', playerPropertyID+'/playersList', unescape(objPlayer.playerName));
        storeSubParameter('sonos-players', playerPropertyID+'/playersListSerial', unescape(objPlayer.serialNum));
        storeSubParameter('sonos-players', playerPropertyID+'/playersListSlaves', unescape(objPlayer.playersInGroup));
        storeSubParameter('sonos-players', playerPropertyID+'/playersListIPSlaves', unescape(objPlayer.playersInGroupIP));
        storeSubParameter('sonos-players', playerPropertyID+'/sonosGroup', unescape(objPlayer.sonosGroup));
        storeSubParameter('sonos-players', playerPropertyID+'/disabled', 'false');
        storeSubParameter('sonos-players', playerPropertyID+'/notDiscovered', 'false');
        storeSubParameter('sonos-players', playerPropertyID+'/deviceID', unescape(objPlayer.playerID));
        
        Property.store();
        
        return playerPropertyID;
    } 
    catch (e) {
        lInfo.logln("Error by saving storePlayer: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
    return INVALID_PLAYER_ID;
}

/** 
 @brief Sets default empty values for action type in PropertyTree
 
 @param sPath [in] - path in PropertyTree
 @param nPropertyID [in] - id of object in PropertyTree
 @param nActionType [in] - scene action type
  
 @return 
*/ 
function setDefaultSceneAction(sPath, nPropertyID, nActionType) {
  switch(nActionType) {
    case SONOS_ACTION_NOW_PLAYING:
      storeSubParameter(sPath, nPropertyID+'/nowAction', -1);
      storeSubParameter(sPath, nPropertyID+'/nowSingleTitle', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleIsRadio', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleArtist', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleUrl', "");
      storeSubParameter(sPath, nPropertyID+'/nowSingleLength', "");
    break;
   
    case SONOS_ACTION_PLAYLIST:
      storeSubParameter(sPath, nPropertyID+'/playListTitle', "");
      storeSubParameter(sPath, nPropertyID+'/playListURL', "");
      storeSubParameter(sPath, nPropertyID+'/playListSonosID', "");
    break;

    case SONOS_ACTION_FAVORITES:
      storeSubParameter(sPath, nPropertyID+'/favoritesTitle', "");
      storeSubParameter(sPath, nPropertyID+'/favoritesURL', "");
      storeSubParameter(sPath, nPropertyID+'/favoritesSonosID', "");
    break;

    case SONOS_ACTION_LIBRARY:
      storeSubParameter(sPath, nPropertyID+'/mediaFileTitle', "");
      storeSubParameter(sPath, nPropertyID+'/mediaFileGUID', "");
      storeSubParameter(sPath, nPropertyID+'/mediaGroupGUID', "");
      storeSubParameter(sPath, nPropertyID+'/mediaGroupSubGUID', "");
    break;

    case SONOS_ACTION_TTS:
      storeSubParameter(sPath, nPropertyID+'/t2Stext', "");
      storeSubParameter(sPath, nPropertyID+'/t2SlangID', "");
      storeSubParameter(sPath, nPropertyID+'/t2SvoiceID', "");
      storeSubParameter(sPath, nPropertyID+'/t2Sguid', "");
    break;
    
    case SONOS_ACTION_COMMAND:
      storeSubParameter(sPath, nPropertyID+'/command', -1);
    break;
  }
}

/** 
 @brief Stores Preset action to PropertyTree
 
 @param objPreset [in] - Object with Preset action data
  
 @return 
*/ 
function storePreset(objPreset) {
    try {
        var nPresetID = parseInt(objPreset.triggerPreset.presetID, 10);
     
        //var presetPropertyID = getDefaultPresetID(objPreset.triggerPreset.roomID, nPresetID);
        var presetPropertyID = objPreset.id;
        var bNewDevice = false; 

        if (presetPropertyID == null) {
          presetPropertyID = getNewIdValue('sonos-presets');
          bNewDevice = true;
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
          setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
        }
        
        var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);
        var nRoomID = parseInt("" + (objPreset.triggerPreset.roomID), 10);
        
        // storing data to propertyTree
        storeSubParameter('sonos-presets', presetPropertyID+'/audioName', '');
        storeSubParameter('sonos-presets', presetPropertyID+'/id', presetPropertyID);
        storeSubParameter('sonos-presets', presetPropertyID+'/roomID', nRoomID);
        storeSubParameter('sonos-presets', presetPropertyID+'/presetID', nPresetID);
        //storeSubParameter('sonos-presets', presetPropertyID+'/presetName', unescape(objPreset.triggerPreset.presetName));
        storeSubParameter('sonos-presets', presetPropertyID+'/volumeSet', unescape(objPreset.whatToPlay.setVolume));
        storeSubParameter('sonos-presets', presetPropertyID+'/volume', unescape(objPreset.whatToPlay.volume));         
        storeSubParameter('sonos-presets', presetPropertyID+'/actionType', nAction);
        storeSubParameter('sonos-presets', presetPropertyID+'/disabled', 'false');
                                   
        switch(nAction){                
          case 0: // now playing
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
            }

            var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
            if (objPreset.whatToPlay.selectedData.changed != null) {
              var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
              storeSubParameter('sonos-presets', presetPropertyID+'/nowAction', nowAction);
              if (bSave) {
              if (nowAction == 0) { // single song/radio
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleIsRadio', objPreset.whatToPlay.selectedData.isRadio);
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
                storeSubParameter('sonos-presets', presetPropertyID+'/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
              }
              else { //active queue
                if (!bNewDevice) { // delete old values
                  var baseNode = Property.getNode('sonos-presets/'+presetPropertyID+'/active-queue');
                  if (baseNode != null) {
                    var oArrayNode = baseNode.getChildren();
                    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                     var oChildNode = oArrayNode[iIndex];
                      baseNode.removeChild(oChildNode.getChild('id').getValue());
                    }
                  }
                }
                
                var baseNode = Property.getNode('sonos-selected-preset');
                var queue = null;
                if (baseNode != null) {
                  queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
                  for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
                    var oQueueItem = queue[iIndex];
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/id", iIndex);
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/title", Utf8.encode(unescape(oQueueItem.title)));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/playID", unescape(oQueueItem.playID));
                    //storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/albumUri", unescape(oQueueItem.albumUri));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/artist", Utf8.encode(unescape(oQueueItem.artist)));
                    storeSubParameter('sonos-presets', presetPropertyID+'/active-queue/'+iIndex+"/trackPos", unescape(oQueueItem.trackPos));
                  }
                }
              }
            }
            }
          break;
          
          case 1: //playlist
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_FAVORITES);
            }
    
            storeSubParameter('sonos-presets', presetPropertyID+'/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-presets', presetPropertyID+'/playListURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-presets', presetPropertyID+'/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;

          case 2: //favorites
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-presets', presetPropertyID, SONOS_ACTION_PLAYLIST);
            }

            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesURL', objPreset.whatToPlay.listUrl);
            storeSubParameter('sonos-presets', presetPropertyID+'/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;
        }
        
        /* Storing trigger structure */
        // Delete old trigger
        var baseNode = Property.getNode('sonos-presets/'+presetPropertyID+'/triggers');
        if (baseNode != null) {
          var oArrayNode = baseNode.getChildren();
          for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
            baseNode.removeChild((iIndex+1));
          }
        }
        
        // add new trigger
        for (iX=1;iX<=4;iX++) {
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+iX+'/type', 'bus-zone-scene');
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+iX+'/scene', nPresetID);
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+iX+'/dsid', -1);

          var nTrgGroupID = 4;
          var nTrgZone = nRoomID;
          switch (iX) {
            case 1:
              nTrgGroupID = 4;
              nTrgZone = nRoomID;
            break;
           
            case 2:
              nTrgGroupID = 4;
              nTrgZone = 0;
            break;

            case 3:
              nTrgGroupID = 0;
              nTrgZone = nRoomID;
            break;

            case 4:
              nTrgGroupID = 0;
              nTrgZone = 0;
            break;
          }
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+iX+'/group', nTrgGroupID);
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+iX+'/zone', nTrgZone);
        }
        storeSubParameter('sonos-selected-preset', 'tmp-queue', "");

        var sPath = Property.getNode('sonos-presets/'+presetPropertyID).getPath();
        unregisterTrigger(sPath);
        registerTrigger(sPath,'sonos_execute',{});
        lInfo.logln("RegTrigger: " + sPath);
        
        storeSubParameter('sonos-presets', presetPropertyID+'/audioName', Utf8.encode(unescape(objPreset.audioName)));
        
        Property.store();
    } 
    catch (e) {
        lInfo.logln("Error by saving storePreset: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
}

/** 
 @brief Stores Activity action to PropertyTree
 
 @param objPreset [in] - Object with Activity action data
  
 @return 
*/ 
function storeActivity(objPreset, bExecutePropertyStore) {
    try {
        var sResultInfo = "";
        var activityPropertyID = objPreset.id;
lInfo.logln("activityPropertyID="+objPreset.id);
        var bNewDevice = false; 
        if (activityPropertyID == null) {
          activityPropertyID = getNewIdValue('sonos-activities');
          bNewDevice = true;
        }
        
        if(bNewDevice) {
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);  
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
          setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
        }   
        else {
          unRegisterActivityTrigger(Property.getNode('sonos-activities/'+activityPropertyID));

          // delete current triggers
          var baseNode = Property.getNode('sonos-activities/'+activityPropertyID+'/triggers');
          if (baseNode != null) {
             var oArrayNode = baseNode.getChildren();
             for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
               baseNode.removeChild((iIndex+1));
             }
          }
          // if there is undo trigger we delete this subbranch
          undoTriggerNode = Property.getNode('sonos-activities/'+activityPropertyID+'/undoCallScene/triggers');
          if (undoTriggerNode != null) {
            Property.getNode('sonos-activities/'+activityPropertyID).removeChild("undoCallScene");
          }
        }
             
        putTriggerParameter(activityPropertyID, objPreset);
        
        var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);
        
        // storing data to propertyTree
        storeSubParameter('sonos-activities', activityPropertyID+'/id', activityPropertyID);
        storeSubParameter('sonos-activities', activityPropertyID+'/volumeSet', unescape(objPreset.whatToPlay.setVolume));
        storeSubParameter('sonos-activities', activityPropertyID+'/volume', unescape(objPreset.whatToPlay.volume));        
        storeSubParameter('sonos-activities', activityPropertyID+'/setRepeat', unescape(objPreset.whatToPlay.setRepeat));
        storeSubParameter('sonos-activities', activityPropertyID+'/repeatTime', unescape(objPreset.whatToPlay.repeatTime));        
        storeSubParameter('sonos-activities', activityPropertyID+'/restartAfter', unescape(objPreset.whatToPlay.restartAfter));
        storeSubParameter('sonos-activities', activityPropertyID+'/createSonosGroup', unescape(objPreset.selectedRoom.createSonosGroup));
        storeSubParameter('sonos-activities', activityPropertyID+'/actionType', nAction);
        storeSubParameter('sonos-activities', activityPropertyID+'/disabled', 'false');
        if (objPreset.isPredefined != null) 
          storeSubParameter('sonos-activities', activityPropertyID+'/predefined', 'true');
        
        // Where to play
        if (objPreset.selectedRoom.action == 0) // all rooms
          storeSubParameter('sonos-activities', activityPropertyID+'/allRooms', 1);        
        else {
          var sRooms = "";
          for ( var iIndex = 0; iIndex < objPreset.selectedRoom.rooms.length; iIndex++) {
            if (iIndex > 0) sRooms += ";"
            sRooms += objPreset.selectedRoom.rooms[iIndex];
          }
         
          storeSubParameter('sonos-activities', activityPropertyID+'/allRooms', 0);        
          storeSubParameter('sonos-activities', activityPropertyID+'/pickedRooms', sRooms);        
        }        
        storeSubParameter('sonos-selected-preset', 'tmp-queue', "");
        
        // What to play
        var bCloudStore = false;
        switch(nAction){
          case SONOS_ACTION_NOW_PLAYING: // now playing
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }
            
            var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
            storeSubParameter('sonos-activities', activityPropertyID+'/nowAction', nowAction);
            if (objPreset.whatToPlay.selectedData.changed != null) {
              var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
              if (bSave) {
              if (nowAction == 0) { // single song/radio
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleIsRadio', unescape(objPreset.whatToPlay.selectedData.isRadio));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
                storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
                if (objPreset.whatToPlay.selectedData.length != null && objPreset.whatToPlay.selectedData.length != "") {
                  var nLength = secondToInt(unescape(objPreset.whatToPlay.selectedData.length));
                  if (!isNaN(nLength))
                    storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleLength', nLength);
                }
              }
              else { //active queue
                if (!bNewDevice) { // delete old values
                  var baseNode = Property.getNode('sonos-activities/'+activityPropertyID+'/active-queue');
                  if (baseNode != null) {
                    var oArrayNode = baseNode.getChildren();
                    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                     var oChildNode = oArrayNode[iIndex];
                      baseNode.removeChild(oChildNode.getChild('id').getValue());
                    }
                  }
                }
                
                var baseNode = Property.getNode('sonos-selected-preset');
                var queue = null;
                if (baseNode != null) {
                  queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
                  for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
                    var oQueueItem = queue[iIndex];
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/id", iIndex);
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/title", Utf8.encode(unescape(oQueueItem.title)));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/playID", unescape(oQueueItem.playID));
                    //storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/albumUri", unescape(oQueueItem.albumUri));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/artist", Utf8.encode(unescape(oQueueItem.artist)));
                    storeSubParameter('sonos-activities', activityPropertyID+'/active-queue/'+iIndex+"/trackPos", unescape(oQueueItem.trackPos));
                  }
                }
              }
            }
            }
          break;

          case SONOS_ACTION_PLAYLIST: //playlist
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-activities', activityPropertyID+'/playListURL', unescape(objPreset.whatToPlay.listUrl));
            storeSubParameter('sonos-activities', activityPropertyID+'/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;

          case SONOS_ACTION_FAVORITES: //favorites
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesURL', objPreset.whatToPlay.listUrl);
            storeSubParameter('sonos-activities', activityPropertyID+'/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
          break;
          
          case SONOS_ACTION_LIBRARY:
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/mediaFileTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
            storeSubParameter('sonos-activities', activityPropertyID+'/mediaGroupGUID', unescape(objPreset.whatToPlay.mediaGroupGUID));
            storeSubParameter('sonos-activities', activityPropertyID+'/mediaGroupSubGUID', unescape(objPreset.whatToPlay.mediaGroupSubGUID));
            storeSubParameter('sonos-activities', activityPropertyID+'/mediaFileGUID', unescape(objPreset.whatToPlay.mediaFileGUID));
            
            // get media file url
            if (objPreset.whatToPlay.mediaFileGUID != '') {
              bCloudStore = true;
              cloudService.fetchMediaFile(unescape(objPreset.whatToPlay.mediaFileGUID), function (oData) {
                if (oData.ReturnMessage == "OK")
                  storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleUrl', unescape(oData.Response.UrlPath));
                if (bExecutePropertyStore)
                  Property.store();
              });
            }
          break;
          
          case SONOS_ACTION_TTS:
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_COMMAND);
            }
            
            var t2sText = Utf8.encode(unescape(objPreset.whatToPlay.t2sText));
            var t2sLang = unescape(objPreset.whatToPlay.t2sLangID);
            var t2sVoice = unescape(objPreset.whatToPlay.t2sVoiceID);
            var t2sFolder = unescape(objPreset.whatToPlay.t2sFolder);

            storeSubParameter('sonos-activities', activityPropertyID+'/t2Stext', t2sText);
            storeSubParameter('sonos-activities', activityPropertyID+'/t2SlangID', t2sLang);
            storeSubParameter('sonos-activities', activityPropertyID+'/t2SvoiceID', t2sVoice);
            
            // if we have cretaed GUID file, we just fetch file url 
            if (objPreset.whatToPlay.t2sGUID != '') {
              bCloudStore = true;
              cloudService.fetchMediaFile(unescape(objPreset.whatToPlay.t2sGUID), function (oData) {
                if (oData.ReturnMessage == "OK") {
                  storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleUrl', unescape(oData.Response.UrlPath));
                  storeSubParameter('sonos-activities', activityPropertyID+'/t2Sguid', unescape(objPreset.whatToPlay.t2sGUID));
                }
                lInfo.logln("Storing inside fetchMediaFile");
                Property.store();
              });
            }
            else
            { // we don't have GUID, we must first create file and then fetch file url
              bCloudStore = true;
              cloudService.createTextSound(t2sFolder, t2sLang, t2sVoice, t2sText, function (oData) {
                if (oData.ReturnMessage == "OK") {
                  lInfo.logln("ACT SAVE - createTextSound="+oData.Response.TextToSpeechVoiceFileGuid);
                  if (oData.Response.TextToSpeechVoiceFileGuid != "") {
                    var fileGuid = unescape(oData.Response.TextToSpeechVoiceFileGuid);
                    cloudService.fetchMediaFile(fileGuid, function (oData) {
                      if (oData.ReturnMessage == "OK") {
                        lInfo.logln("ACT SAVE - Media url="+oData.Response.UrlPath);
                        if (oData.Response.UrlPath != "") {
                          storeSubParameter('sonos-activities', activityPropertyID+'/nowSingleUrl', unescape(oData.Response.UrlPath));
                          storeSubParameter('sonos-activities', activityPropertyID+'/t2Sguid', fileGuid);
                        }
                      } 
                      lInfo.logln("Storing inside fetchMediaFile");
                      Property.store();
                    });
                  }
                  else {
                    lInfo.logln("Storing failed TextToSpeechVoiceFileGuid");
                    Property.store();
                  }
                }
                else {
                  // remembering error code to show info to user 
                  sResultInfo = parseInt("" + oData.ReturnCode);
                  lInfo.logln("Storing failed createTextSound");
                  Property.store();
                }
              });
            }
            
            // we must delete old file
            try {
              lInfo.logln(JSON.stringify(objPreset.whatToPlay));
              if (objPreset.whatToPlay.t2sOriginalGUID != "" && objPreset.whatToPlay.t2sGUID != objPreset.whatToPlay.t2sOriginalGUID) {
                lInfo.logln('DELETING OLD FILE');
                cloudService.deleteTextSound(objPreset.whatToPlay.t2sOriginalGUID, function (oData) {});
              }
            }
            catch (e) {}
          break;

          case SONOS_ACTION_COMMAND: // command
            if (!bNewDevice) {
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_NOW_PLAYING);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_FAVORITES);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_PLAYLIST);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_LIBRARY);
              setDefaultSceneAction('sonos-activities', activityPropertyID, SONOS_ACTION_TTS);
            }

            storeSubParameter('sonos-activities', activityPropertyID+'/command', unescape(objPreset.whatToPlay.command));
          break;
        }
        
         
        if(bExecutePropertyStore) {
          if (!bCloudStore) { 
            lInfo.logln("Normal Activity store");
              Property.store();
          }
        }
        
        //var sPath = Property.getNode('sonos-activities/'+activityPropertyID).getPath();
        //lInfo.logln("Registering trigger started -> "+sPath);
        // unregisterTrigger(sPath);
        //registerTrigger(sPath,'sonos_execute',{});
        var node = Property.getNode('sonos-activities/'+activityPropertyID);
        registerActivityTrigger(node, 'sonos_execute');
        
        //lInfo.logln("Registering trigger finished");
    } 
    catch (e) {
        lInfo.logln("Error by saving storeActivity: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }                        
    }
    return sResultInfo;
}

/** 
 @brief Function is checking in selected room has allready stored predefined triggers
 
 @param roomID [in] - ID of searched room
  
 @return   true if triggers are stored, false if room has no default triggers stored
*/ 
function testIfRoomHasDefaultTriggers(roomID) {
  bTestIfHasDefaultTriggers = false;
  var baseNode = Property.getNode('sonos-presets');
  if (baseNode != null) {
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
      var oChildNode = oArrayNode[iIndex];
        if (oChildNode.getChild('roomID') != null && oChildNode.getChild('roomID').getValue() == roomID) {
          if (JSON.parse(oChildNode.getChild('predefined').getValue())) {
            bTestIfHasDefaultTriggers = true;
            break;   
          }
        }
    }
  }
  return bTestIfHasDefaultTriggers;
}

/** 
 @brief Function creates predefined triggers(predefinedRoomEvents defined in file defined-values.js)
        for selected room
 @param roomID [in] - ID of room to store triggers
  
 @return  
*/ 
function createDefaultRoomPresets(roomID) {
  if (predefinedRoomEvents != null && predefinedRoomEvents.length > 0) {
    for (index in predefinedRoomEvents) {
      var presetPropertyID = getNewIdValue('sonos-presets');
      storeSubParameter('sonos-presets', presetPropertyID+'/id', presetPropertyID);
      storeSubParameter('sonos-presets', presetPropertyID+'/disabled', false);
      storeSubParameter('sonos-presets', presetPropertyID+'/predefined', true);

      storeSubParameter('sonos-presets', presetPropertyID+'/roomID', roomID);
      storeSubParameter('sonos-presets', presetPropertyID+'/presetID', -1);
      //storeSubParameter('sonos-presets', presetPropertyID+'/presetName', "");
      storeSubParameter('sonos-presets', presetPropertyID+'/actionType', SONOS_ACTION_COMMAND);
      storeSubParameter('sonos-presets', presetPropertyID+'/command', index);

      storeSubParameter('sonos-presets', presetPropertyID+'/volumeSet', false);
      storeSubParameter('sonos-presets', presetPropertyID+'/volume', 0);         

      storeSubParameter('sonos-presets', presetPropertyID+'/audioName', "");
      storeSubParameter('sonos-presets', presetPropertyID+'/restartAfter', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/setRepeat', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/allRooms', false);         
      storeSubParameter('sonos-presets', presetPropertyID+'/repeatTime', 0);         
      storeSubParameter('sonos-presets', presetPropertyID+'/pickedRooms', 0);         
      
      var arrayTriggers = predefinedRoomEvents[index];
      var trgSubIndex = 0;
      for(trgIndx in arrayTriggers) {
        lInfo.logln("trigger"+arrayTriggers[trgIndx].id+" -> "+presetPropertyID);
        /*
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/type', 'bus-zone-scene');
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/zone', roomID);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/group', arrayTriggers[trgIndx].group);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/scene', arrayTriggers[trgIndx].id);
        storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgIndx+'/dsid', -1);
        */
        
        var nSubTriggers = 4;
        if (arrayTriggers[trgIndx].group == 0) // for Scenes allready assigned groupID as 0 we skip other sub triggers registrstration
          nSubTriggers = 1;
          
        for (iX=1;iX<=nSubTriggers;iX++) {
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgSubIndex+'/type', 'bus-zone-scene');
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgSubIndex+'/scene', arrayTriggers[trgIndx].id);
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgSubIndex+'/dsid', -1);

          var nTrgGroupID = arrayTriggers[trgIndx].group;
          var nTrgZone = roomID;
          switch (iX) {
            case 1:
              nTrgGroupID = arrayTriggers[trgIndx].group;
              nTrgZone = roomID;
            break;
           
            case 2:
              nTrgGroupID = arrayTriggers[trgIndx].group;
              nTrgZone = 0;
            break;

            case 3:
              nTrgGroupID = 0;
              nTrgZone = roomID;
            break;

            case 4:
              nTrgGroupID = 0;
              nTrgZone = 0;
            break;
          }
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgSubIndex+'/group', nTrgGroupID);
          storeSubParameter('sonos-presets', presetPropertyID+'/triggers/'+trgSubIndex+'/zone', nTrgZone);
          trgSubIndex++;
        }
      }
      var sPath = Property.getNode('sonos-presets/'+presetPropertyID).getPath();
      unregisterTrigger(sPath);
      registerTrigger(sPath,arrayTriggers[trgIndx].trigger,{});
      lInfo.logln("RegTrigger: " + sPath);
    } 
    Property.store();
    /*{
      // add new trigger
    } */
  } 
}

/** 
 @brief Function assigns player to dss Room
 
 @param id [in] - player ID to assign room
 @param roomID [in] - ID of room 
  
 @return  
*/ 
function assignPlayerRoom(id, nRoomID) {
    try {
        var bCreatedDefaultPresets = false;
        var roomID = parseInt("" + unescape(nRoomID), 10);
     
        storeSubParameter('sonos-players', id+'/roomID', roomID);
        
        if (!testIfRoomHasDefaultTriggers(roomID)) {
          createDefaultRoomPresets(roomID);
          bCreatedDefaultPresets = true;
        }
        
        storeSubParameter('sonos-players', id+'/roomID', roomID);
        
        //storeSubParameterFull('/apartment/'+roomID+'/groups/group4/connectedDevices', 1);
        updateZoneConnectedDevices();
        
        Property.store();
        return bCreatedDefaultPresets;
    } 
    catch (e) {
        lInfo.logln("Error by saving assignPlayerRoom: " + e);
        for ( var i in e) {
            lInfo.logln(i + ': ' + e[i]);
        }
    }
}

/** 
 @brief Function retrives full list of sonos devices from Sonos call http://ip:port//status/topology
        and stores them as players in PT
 
 @param sSonosDeviceIP [in] - IP of sonos device
  
 @return  true if players are found and stored
*/ 
function getSonosDevices(sSonosDeviceIP) {
  var curl_options = new Object({'debug' : 0, 'connectTimeout' : URL_CONNECT_TIMEOUT});
try {        
  var h = new HTTP(curl_options, '');
  var result = h.get('http://'+sSonosDeviceIP+':'+SONOS_PORT+'/status/topology', '');
  if (result != null && result.body != null) {
    var aList = getXMLFields(result.body, 'ZonePlayer'); 
    
    var mainDevices = new Array();
    var subDevices = new Array();
    
  lInfo.logln("length="+aList.length);

    for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
      var sPlayerID = "";
      var bCoordinator = false;
      var sLocation = "";
      var sDeviceIP = ""; 
      var sPlayerSerial = "";
      var sPlayerName = "";
      var sRoomName = "";
      var sGroupID = "";

      var sData = aList[iIndex].data; 
      lInfo.logln("pos:"+iIndex);

      if (sData != null) {
        matches = sData.match(/location=\'(.*?)\'/); 
        if (matches != null) {
          sLocation = matches[1];     
     
          // check if device is speaker
          var tmpData = h.get(sLocation);
          if (tmpData != null && tmpData.body!= null && tmpData.body.indexOf("urn:schemas-upnp-org:service:MusicServices")>0) {
            var matches = sLocation.match(/:\/\/(.[^:]+)/); // get ip from full url
            if (matches != null) 
              sDeviceIP = matches[1]; 
      
            var matches = sData.match(/uuid=\'(.*?)\'/); 
            if (matches != null) 
              sPlayerID = matches[1]; 
            
            matches = tmpData.body.match(/<modelName>(.*?)<\/modelName>/);  
            if (matches != null) 
              sPlayerName = matches[1];   
                        
            matches = tmpData.body.match(/<serialNum>(.*?)<\/serialNum>/);  
            if (matches != null) 
              sPlayerSerial = matches[1];   
              
            matches = sData.match(/group=\'(.*?)\'/); 
            if (matches != null) {
              sGroupID = matches[1];     
            }

            matches = sData.match(/coordinator=\'(.*?)\'/); 
            if (matches != null) {
              bCoordinator = (matches[1].toLowerCase() == "true");     
            }
          
            matches = sData.match(/<ZonePlayer[^>]*>(.*?)<\/ZonePlayer>/); 
            if (matches != null) 
              sRoomName = matches[1];     

            if (bCoordinator) {
              mainDevices.push({playerID: sPlayerID,
                                playerName:sPlayerName, 
                                serialNum: sPlayerSerial, 
                                playersInGroup: "",
                                playersInGroupIP: "",
                                sonosRoom: sRoomName,
                                sonosGroup: sGroupID,
                                roomID: 0,
                                deviceIP: sDeviceIP});
            } 
            else {
              subDevices.push({playerID: sPlayerID,
                               playerName:sPlayerName, 
                               serialNum: sPlayerSerial,
                               playersInGroup: "",
                               sonosRoom: sRoomName,
                               sonosGroup: sGroupID,
                               deviceIP: sDeviceIP});
            }
              
            lInfo.logln("Found device: " + sPlayerID + " * "+sDeviceIP+" * "+sPlayerSerial+" * "+sRoomName + " * " + sGroupID);
          }
        }
      }
    } 
    
    var bDevicesFound = false;
    
    // Storing new devices to Property tree
    if (mainDevices.length > 0) {
      bDevicesFound = true;
     
      //var roomsList = new Array();   
      var arrayAssignedRooms = new Array();
      
      var baseNode = Property.getNode('sonos-players');
      if (baseNode != null) {
        var oArrayNode = baseNode.getChildren();
        for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          lInfo.logln("player:"+iIndex);
          var oChildNode = oArrayNode[iIndex];
          /*
          if (oChildNode.getChild('sonosGroup') != null && oChildNode.getChild('sonosGroup').getValue() != null) {
            if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() > 0)  {
              roomsList.push({groupID:oChildNode.getChild('sonosGroup').getValue(), 
                              roomID:oChildNode.getChild('roomID').getValue()
                             });
            }
          }
          */

          var nRoomID = parseInt("" + oChildNode.getChild('roomID').getValue());
          
          if (!JSON.parse(oChildNode.getChild('disabled').getValue()) && nRoomID > 0) {
            arrayAssignedRooms.push({'deviceID':unescape(oChildNode.getChild('deviceID').getValue()), 'roomID': nRoomID});
          }
          
          storeSubParameter('sonos-players', oChildNode.getChild('id').getValue()+'/notDiscovered', 'true');
          storeSubParameter('sonos-players', oChildNode.getChild('id').getValue()+'/disabled', 'true');
        }
      }
      
      /*  
      // Delete old devices
      var baseNode = Property.getNode('sonos-players');
      if (baseNode != null) {
           var oArrayNode = baseNode.getChildren();
           for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
             var oChildNode = oArrayNode[iIndex];
             storeSubParameter('sonos-players', oChildNode.getChild('id').getValue()+'/notDiscovered', 'true');
             storeSubParameter('sonos-players', oChildNode.getChild('id').getValue()+'/disabled', 'true');
             //baseNode.removeChild();
           }
      }
      */
      
      for ( var iIndex = 0; iIndex < mainDevices.length; iIndex++) {
        // search if has any sub devices
        var sSubDevices = ""; 
        var sSubDevicesSer = ""; 
        var sSubDevicesGroup = "";
        var sSubDevicesGroupIP = "";
        var sSubRooms = ""; 
        for ( var yIndex = 0; yIndex < subDevices.length; yIndex++) {
           if (subDevices[yIndex].sonosGroup == mainDevices[iIndex].sonosGroup) {
             if (sSubDevices.length > 0) 
               sSubDevices = sSubDevices + ", ";
             sSubDevices = sSubDevices + subDevices[yIndex].playerName;

             if (sSubDevicesSer.length > 0) 
               sSubDevicesSer = sSubDevicesSer + ", ";
             sSubDevicesSer = sSubDevicesSer + subDevices[yIndex].serialNum;

             if (sSubDevicesGroup.length > 0) 
               sSubDevicesGroup = sSubDevicesGroup + ", ";
             sSubDevicesGroup = sSubDevicesGroup + subDevices[yIndex].playerID;
             
             if (sSubDevicesGroupIP.length > 0) 
               sSubDevicesGroupIP = sSubDevicesGroupIP + ", ";
             sSubDevicesGroupIP = sSubDevicesGroupIP + subDevices[yIndex].deviceIP;
             
             
             if (subDevices[yIndex].sonosRoom != mainDevices[iIndex].sonosRoom) {
               if (sSubRooms.length > 0) 
                 sSubRooms = sSubRooms + ", ";
               sSubRooms = sSubRooms + subDevices[yIndex].sonosRoom;
             }
           }
        }
        
        if (sSubDevices.length > 0)  {
          mainDevices[iIndex].playerName = mainDevices[iIndex].playerName + ", " + sSubDevices;
        }

        if (sSubDevicesSer.length > 0)  {
          mainDevices[iIndex].serialNum = mainDevices[iIndex].serialNum + ", " + sSubDevicesSer;
        }

        mainDevices[iIndex].playersInGroup = "";
        if (sSubDevicesGroup.length > 0)  {
          mainDevices[iIndex].playersInGroup = sSubDevicesGroup;
        }

        mainDevices[iIndex].playersInGroupIP = "";
        if (sSubDevicesGroupIP.length > 0)  {
          mainDevices[iIndex].playersInGroupIP = sSubDevicesGroupIP;
        }
        
        if (sSubRooms.length > 0)  {
          mainDevices[iIndex].sonosRoom = "Grouped ["+mainDevices[iIndex].sonosRoom + ", " + sSubRooms+"]";
        }
        
        lInfo.logln("Search group "+mainDevices[iIndex].sonosGroup)
        //mainDevices[iIndex].roomID = getStoredRoom(roomsList, mainDevices[iIndex].sonosGroup);
        
        var playerID = storePlayer(mainDevices[iIndex], arrayAssignedRooms);
        if (playerID != INVALID_PLAYER_ID)
          resetStates(playerID, 0);
      } 
    }
    
    return (mainDevices.length > 0);
  }
 }
 catch (e) {
      lInfo.logln("Error executing getSonosDevices " + e);
 }
 return false;
}

/** 
 @brief Function creates all triggers for Preset and Activities action
 
 @param -
  
 @return  
*/ 
function updateAllTriggers(){
    var baseNode = Property.getNode('sonos-presets');
    if (baseNode != null) {
      var oArrayNode = baseNode.getChildren();
      if (oArrayNode != null) {
        for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          var sPath = oChildNode.getPath();
          lInfo.logln("sonos-presets Create Trigger["+iIndex+"] " + sPath);
          
          var sTriggerType = 'sonos_execute';
          var fPredefined=false;
          if (oChildNode.getChild('predefined')!=null)
        	  fPredefined=JSON.parse(oChildNode.getChild('predefined').getValue());
          if ((fPredefined) &&
                  (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND) &&
                  (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_PAUSE)
                 ) {
                 sTriggerType = 'sonos_execute_pause';
              }
          if ((fPredefined) &&
                  (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND) &&
                  (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_VOLUME_UP)
                 ) {
                 sTriggerType = 'sonos_execute_volume';
              }
          if ((fPredefined) &&
                  (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND) &&
                  (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_VOLUME_DOWN)
                 ) {
                 sTriggerType = 'sonos_execute_volume';
              }
          registerTrigger(oChildNode.getPath(), sTriggerType,{});
        }
      }
    }

    var baseNode = Property.getNode('sonos-activities');
    if (baseNode != null) {
      var oArrayNode = baseNode.getChildren();
      if (oArrayNode != null) {
        for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          var sPath = oChildNode.getPath();
          lInfo.logln("sonos-activities Create Trigger["+iIndex+"] " + sPath);
          registerActivityTrigger(oChildNode, 'sonos_execute');
        }
      }
    }
}

/** 
 @brief Function creates predefined triggers(predefinedStopActivities defined in file defined-values.js)
        for all activities
        
 @param roomID [in] - ID of room to store triggers
  
 @return  
*/ 
function preparePredefinedActivities() {
    var bPropertySaveExecuted = false;

    var baseNode = Property.getNode('sonos-predifined/activities');
    var bFillPredefined = true;
    var bDeleteOldPredefined = false;
    if (baseNode != null) { // activities allready added 
      if (baseNode.getValue() > 2) // predefined latest version 
        bFillPredefined = false; 
      else {
        // we need to delete old predefined
        bDeleteOldPredefined = true
      }
    }
    
    if (bFillPredefined) 
    { 
      if (Property.getNode('/scripts/remote-connectivity/OSPToken') != null) {
        var sLangCode = Property.getProperty('/system/language/locale');
        lInfo.logln("sonos3"+sLangCode);
       
        // remember all stored activities triggers, if trigger exists we skip creation of activity
        var arrayStored = new Array();
        arrayStored['bus-zone-scene'] = new Array();
        arrayStored['state-change'] = new Array();
        var baseNode = Property.getNode('sonos-activities');
        if (baseNode != null) {
          var oArrayNode = baseNode.getChildren();
          for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
            var oChildNode = oArrayNode[iIndex];
            if (oChildNode.getChild('triggers') != null) {
              var oTriggers = oChildNode.getChild('triggers').getChildren();
              if (oTriggers != null) {
                for ( var yIndex = 0; yIndex < oTriggers.length; yIndex++) {
                   var oChildTrigger = oTriggers[yIndex];
                   var nType = oChildTrigger.getChild('type').getValue();
                   switch(nType) {
                      case 'bus-zone-scene':
                        arrayStored['bus-zone-scene'].push(new Object({'id':oChildNode.getChild('id').getValue(),
                                                                       'scene':oChildTrigger.getChild('scene').getValue()})); 
                      break;
                      
                      case 'state-change':
                        arrayStored['state-change'].push(new Object({'id':oChildNode.getChild('id').getValue(),
                                                                     'name':oChildTrigger.getChild('name').getValue(),
                                                                     'state':oChildTrigger.getChild('state').getValue()})); 
                      break;
                   }
                }
              }
            }
          }
        }       
       
        storeSubParameter('sonos-predifined', 'activities', 3); // stored predefined and needs to be localized
       
        var arrayActivitiesToStore = new Array();
        for (var yIndex=0;yIndex<predefinedActivities.length;yIndex++) {
          var command = predefinedActivities[yIndex];
          var bStore = true;
          switch(command.type) {       
              case 'bus-zone-scene':
                if (arrayStored['bus-zone-scene'] != null) {
                  for(var zIndex=0;zIndex<arrayStored['bus-zone-scene'].length;zIndex++) {
                    if (arrayStored['bus-zone-scene'][zIndex] != null && 
                        arrayStored['bus-zone-scene'][zIndex].scene == command.scene) {
                      if (bDeleteOldPredefined) {
                        unRegisterActivityTrigger(Property.getNode('sonos-activities/'+arrayStored['bus-zone-scene'][zIndex].id));
                        baseNode.removeChild(arrayStored['bus-zone-scene'][zIndex].id);
                      }
                      else
                        bStore = false;
                      break;
                    }
                  }
                }
              break;
              
              case 'state-change':
                if (arrayStored['state-change'] != null){
                  for(var zIndex=0;zIndex<arrayStored['state-change'].length;zIndex++) {
                    if (arrayStored['state-change'][zIndex] != null && 
                        arrayStored['state-change'][zIndex].name == command.name &&
                        arrayStored['state-change'][zIndex].state == command.state) {
                      if (bDeleteOldPredefined) {
                        unRegisterActivityTrigger(Property.getNode('sonos-activities/'+arrayStored['state-change'][zIndex].id));
                        baseNode.removeChild(arrayStored['state-change'][zIndex].id);
                      }
                      else
                        bStore = false;
                      break;    
                    }
                  }
                }
              break;
          }
          
          if (bStore) {
            // Store predefined Activity
            var objActivity = new Object();
            objActivity.id = null;
            objActivity.triggers = new Array();
            objActivity.triggers.push(command);
            objActivity.whatToPlay = new Object();
            objActivity.whatToPlay.actionType = command.actionType;
            objActivity.whatToPlay.setVolume = 0;
            objActivity.whatToPlay.volume = 0;
            objActivity.whatToPlay.setRepeat = 0;
            objActivity.whatToPlay.repeatTime = 0;        
            objActivity.whatToPlay.restartAfter = 0;
            objActivity.selectedRoom = new Object();
            objActivity.selectedRoom.action = 0; //all rooms
            objActivity.selectedRoom.createSonosGroup = 0;
            objActivity.isPredefined = 1;
            
            switch(command.actionType){
              case SONOS_ACTION_LIBRARY:
                if (command.volume > -1) {      
                  objActivity.whatToPlay.setVolume = true;
                  objActivity.whatToPlay.volume = command.volume;
                }
                objActivity.selectedRoom.createSonosGroup = command.groupRooms;
                if (command.repeat > 0) {      
                  objActivity.whatToPlay.setRepeat = true;
                  objActivity.whatToPlay.repeatTime = command.repeat;        
                }
                objActivity.whatToPlay.restartAfter = command.restart;
            
                objActivity.whatToPlay.title = "";
                if (command.title != null)
                  objActivity.whatToPlay.title = command.title;
               
                var sFileName = "";  
                for (iIndex=0;iIndex<command.file.length;iIndex++) {
                  if (iIndex==0) // if lang is not found, first file is set
                    sFileName = command.file[iIndex].guid;
                  
                  var keepGoing = true;
                  switch(sLangCode) {  //
                    case "en_US":
                    case "en_GB":
                    case "en_EN":
                      if (command.file[iIndex].lang == "EN") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;
                    
                    case "de_DE":
                    case "de_CH":
                    case "de_AT":
                      if (command.file[iIndex].lang == "DE") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;
                    
                    case "fr_FR":
                      if (command.file[iIndex].lang == "FR") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;
                    
                    case "nl_NL":
                      if (command.file[iIndex].lang == "NL") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;

                    case "nl_TR":
                      if (command.file[iIndex].lang == "TR") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;

                    case "nl_IT":
                      if (command.file[iIndex].lang == "IT") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;
                          
                    case "pl_PL":
                      if (command.file[iIndex].lang == "PL") {
                        sFileName = command.file[iIndex].guid;
                        keepGoing = false;
                      }
                    break;
                  }
                  
                  if (!keepGoing) break;
                }
                objActivity.whatToPlay.mediaFileGUID = sFileName;

                objActivity.whatToPlay.mediaGroupGUID = command.groupID;
                objActivity.whatToPlay.mediaGroupSubGUID = "";
                if (command.subGroupID != null)
                  objActivity.whatToPlay.mediaGroupSubGUID = command.subGroupID;
              break;
                
              case SONOS_ACTION_COMMAND: 
                objActivity.whatToPlay.command = command.command;
              break;
            }
            
            arrayActivitiesToStore.push(objActivity);
          } 
          else {
            lInfo.logln("Skipping predefined Activity = "+ command.title);
          }
        }
        
        for (var yIndex=0;yIndex<arrayActivitiesToStore.length;yIndex++) {
          bStorePT = false;
          if (yIndex == (arrayActivitiesToStore.length-1)) {
            bStorePT = true; // we store PT only on last predefined activity
            bPropertySaveExecuted = true;
          }
          
          storeActivity(arrayActivitiesToStore[yIndex], bStorePT);
          
        }
      }
    }
    
    return bPropertySaveExecuted;
}

function updateZoneConnectedDevices() {
  //connectedDevices
  var zones = getZones();
  for ( var iIndex = 0; iIndex < zones.length; iIndex++) {
    if (zones[iIndex].id === 0) {
      continue;
    }
    try {
      zones[iIndex].removeConnectedDevice(4);
    }
    catch (e)
    {
      storeSubParameterFull('/apartment/zones/zone'+zones[iIndex].id+'/groups/group4/connectedDevices', 0);
    }
  }
  
  baseNode = Property.getNode('sonos-players');
  if (baseNode != null) {
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
      var oChildNode = oArrayNode[iIndex];
      if (!JSON.parse(oChildNode.getChild('notDiscovered').getValue())) {
        if (oChildNode.getChild('roomID') != null) {
          var nRoomID = parseInt("" + (oChildNode.getChild('roomID').getValue()), 10);
          if (nRoomID > 0) {
            lInfo.logln("Activate connectedDevices for zoneID="+nRoomID);
            try {
              getZoneByID(nRoomID).addConnectedDevice(4);
            }
            catch (e)
            {
              storeSubParameterFull('/apartment/zones/zone'+nRoomID+'/groups/group4/connectedDevices', 1);
            }
          }
        }
      }
    }
  }
}

function startCloudService(bRetryConnection) {
    try {
      lInfo.logln('start startService');
      
      if (Property.getNode('/scripts/remote-connectivity/OSPToken')==null) {
        if (bRetryConnection) {
          lInfo.logln('Remote Connectivity not ready! New try in 5 minutes.');
          var evt = new Event("sonos-controller.save",{actions:'startService','time':'+300'});      
          evt.raise();  
        }
        return;
      }
      else {
        cloudService.startService(function (oData) {
          if (oData.ReturnMessage != "OK") {
            if (bRetryConnection) {
              lInfo.logln('StartMediaService failed! New try in 5 minutes.');
              var evt = new Event("sonos-controller.save",{actions:'startService','time':'+300'});      
              evt.raise();  
            }
          }
          else
          {
            lInfo.logln('StartMediaService is connected.');
          }
        });
      }
    } catch (e) {
      lWarn.logln('startService error ' + e);
    }
}

function deleteUnusedSonosQueues(sSonosIP) {
  try {
    // deleting all temporary created playlists
    var curl_options = new Object({
          'debug' : 1,
          'connectTimeout' : 2000
    });
    var arrayLists = upnpGetSonosPlaylist(sSonosIP, 1000, true, curl_options);
    if (arrayLists != null && arrayLists.length > 0) {
      for (var iIndex=0;iIndex<arrayLists.length;iIndex++) {
        upnpDeleteQueueFromList(sSonosIP, arrayLists[iIndex].listID, curl_options);
      }
    }
  } catch (e) {
    lWarn.logln('deleteUnusedQueues error ' + e);
  }
}             


if (raisedEvent.name === 'running') {
    lInfo.logln("running sonos players.js");
    Property.load();
    lInfo.logln("running sonos players.js end");
}
else
  if (raisedEvent.name == 'model_ready') {
    lInfo.logln("model_ready sonos players.js");

    // start cloud service
    startCloudService(true);
    
    updateAllTriggers();  
    
    updateZoneConnectedDevices();
    
    var sActiveSonosIP = "";
    var baseNode = Property.getNode('sonos-players');
    if (baseNode != null) {
      var oArrayNode = baseNode.getChildren();
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        resetStates(oChildNode.getChild('id').getValue(), 0);
        if (!JSON.parse(oChildNode.getChild('disabled').getValue()) && sActiveSonosIP == "")
          sActiveSonosIP = oChildNode.getChild('ipAddress').getValue();
      }
    }
    
    storeSubParameter('sonos-grouping', 'data', '');
    storeSubParameter('sonos-grouping', 'sceneID', '');
    
    if (!preparePredefinedActivities())
      Property.store();

    if (sActiveSonosIP != "")
      deleteUnusedSonosQueues(sActiveSonosIP);

    lInfo.logln("model_ready sonos players.js end");
  }
else
  if (raisedEvent.name == 'sonos-controller.save') {
    if (raisedEvent.parameter.actions == 'assignRoom') {
       var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
       lInfo.logln("Assign room:"+sObject.playersList.roomID); 
       var bCreateDefaultPresets = false;
       if (sObject != null && sObject.playersList.roomID > 0)
         bCreateDefaultPresets = assignPlayerRoom(sObject.id, sObject.playersList.roomID);
         
       nAssignedRoom = -1;
       if (bCreateDefaultPresets)
         nAssignedRoom = sObject.playersList.roomID; 

       var evt = new Event("sonos-controller-saved", {nNewRoomID:nAssignedRoom});
       evt.raise();
    }
    if (raisedEvent.parameter.actions == 'deletePreset') {
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));

        if (typeof(oIDObject)!='object') {
          oIDObject=[oIDObject];
        }
        
        for (var sKey in oIDObject) {
          var baseNode = Property.getNode("sonos-presets");
          if (baseNode != null) {
             var sPath = Property.getNode('sonos-presets/'+oIDObject[sKey]).getPath();
             lInfo.logln("Delete trigger " + sPath);
             unregisterTrigger(sPath);
             lInfo.logln("Delete trigger finished");
             baseNode.removeChild(oIDObject[sKey]);
          }
        }
        Property.store();

        var evt = new Event("sonos-controller-saved",  {});
        evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'deleteActivity') {
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        
        if (typeof(oIDObject)!='object') {
          oIDObject=[oIDObject];
        }
        
        for (var sKey in oIDObject) {
          var baseNode = Property.getNode("sonos-activities");
          if (baseNode != null) {
             var sPath = Property.getNode('sonos-activities/'+oIDObject[sKey]).getPath();
             lInfo.logln("Delete trigger " + sPath);
             unRegisterActivityTrigger(Property.getNode('sonos-activities/'+oIDObject[sKey]));
             lInfo.logln("Delete trigger finished");

             lInfo.logln("Delete child " + oIDObject[sKey]);
             baseNode.removeChild(oIDObject[sKey]);
          }
        }
        Property.store();
          
        var evt = new Event("sonos-controller-saved",  {});
        evt.raise();
        lInfo.logln("sonos-controller-saved raised");
    }
    else
    if (raisedEvent.parameter.actions == 'storePreset') {
       var objData = JSON.parse(unescape(raisedEvent.parameter.value));
       if (objData != null) {
         lInfo.logln("Store preset"); 
         storePreset(objData);
       }
       var evt = new Event("sonos-controller-saved",  {});
       evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'storeActivity') {
       var objData = JSON.parse(unescape(raisedEvent.parameter.value));
       var sInfo = "";
       if (objData != null) {
         lInfo.logln("Store activity"); 
         sInfo = storeActivity(objData, true);
       }
       lInfo.logln("Store activity create saved event ["+sInfo+"]");  
       var evt = new Event("sonos-controller-saved", {value:JSON.stringify(sInfo)});
       evt.raise();
    }
    else
    if (raisedEvent.parameter.actions == 'enablePreset') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPresetID = parseInt("" + oIDObject[sKey],10);
        if (nPresetID > -1) {  
         storeSubParameter('sonos-presets', nPresetID+'/disabled', false);
         var sPath = Property.getNode('sonos-presets/'+nPresetID).getPath();
         registerTrigger(sPath,'sonos_execute',{});
lInfo.logln("Enable Preset "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disablePreset') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPresetID = parseInt("" + oIDObject[sKey],10);
        if (nPresetID > -1) {  
         storeSubParameter('sonos-presets', nPresetID+'/disabled', true);
         var sPath = Property.getNode('sonos-presets/'+nPresetID).getPath();
         unregisterTrigger(sPath);
lInfo.logln("Disabled Preset "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
    else
    if (raisedEvent.parameter.actions == 'enableActivity') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nActivityID = parseInt("" + oIDObject[sKey],10);
        if (nActivityID > -1) {  
         storeSubParameter('sonos-activities', nActivityID+'/disabled', false);
         var sPath = Property.getNode('sonos-activities/'+nActivityID).getPath();
         registerActivityTrigger(Property.getNode('sonos-activities/'+nActivityID), 'sonos_execute');
lInfo.logln("Enable Activity "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disableActivity') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nActivityID = parseInt("" + oIDObject[sKey],10);
        if (nActivityID > -1) {  
         storeSubParameter('sonos-activities', nActivityID+'/disabled', true);
         var sPath = Property.getNode('sonos-activities/'+nActivityID).getPath();
         unregisterTrigger(sPath);
         unRegisterActivityTrigger(Property.getNode('sonos-activities/'+oIDObject[sKey]));
lInfo.logln("Disabled Activity "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
    else
    if (raisedEvent.parameter.actions == 'enablePlayer') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPlayerID = parseInt("" + oIDObject[sKey],10);
        if (nPlayerID > -1) {  
         storeSubParameter('sonos-players', nPlayerID+'/disabled', false);
lInfo.logln("Enabled Player "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();  
    }
    else
    if (raisedEvent.parameter.actions == 'disablePlayer') {
      var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
      if (typeof(oIDObject)!='object')
          oIDObject=[oIDObject];

      for (var sKey in oIDObject) {
        nPlayerID = parseInt("" + oIDObject[sKey],10);
        if (nPlayerID > -1) {  
         storeSubParameter('sonos-players', nPlayerID+'/disabled', true);
lInfo.logln("Disabled Player "+sPath);         
         Property.store();
        }
      }
      var evt = new Event("sonos-controller-saved", {});
      evt.raise();     
    }
    else if (raisedEvent.parameter.actions == 'startService') {
      startCloudService(true);
    }
  }
  else
    if (raisedEvent.name == 'sonos-controller.startdiscovery') {
      if (raisedEvent.parameter.actions == 'discover') {
      var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
lInfo.logln("startdiscovery="+sObject.discoverIP);
      var sResult = "";
      if (getSonosDevices(sObject.discoverIP)) {
        sResult = sObject.discoverIP;

        updateZoneConnectedDevices();
        
        storeSubParameter('sonos-grouping', 'data', '');
        storeSubParameter('sonos-grouping', 'sceneID', '');
        if (!preparePredefinedActivities())
          Property.store();
      }
      
      var evt = new Event("sonos-discover-finished",  {ip:sResult});
      evt.raise(); 
      lInfo.logln("End discovery");
    }
    else if (raisedEvent.parameter.actions == 'AutoDiscover') {
        var arrayAdresses = JSON.parse(unescape(raisedEvent.parameter.value));
        var curl_options = new Object({
          'debug' : 0,
          'connectTimeout' : URL_CONNECT_TIMEOUT
        });
        lInfo.logln("AutoDiscover "+unescape(raisedEvent.parameter.value));        
        var h = new HTTP(curl_options, '');
        for ( var iIndex = 0; iIndex < arrayAdresses.length; iIndex++) {
          var oChild = arrayAdresses[iIndex];
          var sIP = oChild.ipAddress;
          
          if (oChild.inList == 1)  { // testing on existing IP from grid
            lInfo.logln("Autodiscovery on existing IP "+sIP);
           
            var sUrl = "http://"+sIP+":"+SONOS_PORT+"/xml/device_description.xml";
            lInfo.logln("Testurl "+sUrl);

            var result = h.get(sUrl);
            if (result != null && result.body != null)
            {
              lInfo.logln("Result: IP returns result");
              if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
                lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
                if (getSonosDevices(sIP)) {
                  lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
                  sResult = sIP;
                  lInfo.logln("Create predefined activities");
                  updateZoneConnectedDevices();
                  
                  storeSubParameter('sonos-grouping', 'data', '');
                  storeSubParameter('sonos-grouping', 'sceneID', '');
                  if (!preparePredefinedActivities())
                    Property.store();
                  
                  break;          
                }
              }
            }
          }
          else {  // testing dSS IP and range 1-255
            var aIP = sIP.split('.');
            if (aIP.length == 4)
              sIP = aIP[0]+"."+aIP[1]+"."+aIP[2]+".";
            
            lInfo.logln("Testing dSS-IP range "+sIP);
          
            var sResult = "";
            for ( var iIndex = 1; iIndex < 256; iIndex++) {
              var sSearchedIP = sIP+iIndex;
             
              var sUrl = "http://"+sSearchedIP+":"+SONOS_PORT+"/xml/device_description.xml";
              lInfo.logln("Testurl "+sUrl);

              var result = h.get(sUrl);
              if (result != null && result.body != null)
              {
                if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
                  lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
                  if (getSonosDevices(sSearchedIP)) {
                    lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
                    sResult = sIP;
                    updateZoneConnectedDevices();
                    
                    storeSubParameter('sonos-grouping', 'data', '');
                    storeSubParameter('sonos-grouping', 'sceneID', '');
                    if(!preparePredefinedActivities())
                      Property.store();
                    break;          
                  }
                }
              }
              result = null;
              lInfo.logln("----------");
            }
          }

          lInfo.logln("IP:"+ oChild.ipAddress + " / inList:" + oChild.inList);
        }
        lInfo.logln("Raising event sonos-autodiscover-finished");
        var evt = new Event("sonos-autodiscover-finished",  {ip:sResult});
        evt.raise(); 
    }
  }
  else
    if (raisedEvent.name == 'sonos-controller.getdata') {
         if (raisedEvent.parameter.actions == 'getSonosQueue') { 
           lInfo.logln("Inside getSonosQueue");

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000 /* in mseconds */
            });
             
           var arrayLists = upnpGetCurrentQueue(oObject, 100, curl_options);
           
           var data = JSON.stringify(arrayLists);     
           lInfo.logln(data);
           storeSubParameter('sonos-selected-preset', 'tmp-queue', escape(data));
           Property.store();
           
           
           var evt = new Event("sonos-discover-return-sonos-queue", {returnValue : data});
           evt.raise();
         }
         else
         if (raisedEvent.parameter.actions == 'getSonosTrack') { 

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
           lInfo.logln("Inside getSonosTrack for IP:"+oObject);
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000 /* in mseconds */
            });
                                                       
           var objCurrentTrack = upnpGetCurrentTrack(oObject, curl_options);
           var data = JSON.stringify(objCurrentTrack);
           lInfo.logln(data);
           var evt = new Event("sonos-discover-return-sonos-track", {returnValue : data});
           evt.raise();
         }
         else
         if (raisedEvent.parameter.actions == 'getSonosPlaylists') { 
           lInfo.logln("Inside getSonosPlaylists"+raisedEvent.parameter.value);

           var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
                            
           var curl_options = new Object({
                'debug' : 1,
                'connectTimeout' : 2000
           });
           if (oObject.sonosIP != "") {
              var arrayLists = upnpGetSonosPlaylist(oObject.sonosIP, NR_ITEMS_FETCHED_FROM_PLAYLIST, false, curl_options);
              saveSonosPlayList(arrayLists);
              
              arrayLists = upnpGetSonosFavorites(oObject.sonosIP, NR_ITEMS_FETCHED_FROM_FAVORITES, curl_options);
              saveSonosFavorites(arrayLists);
           }
           
           if (oObject.loadMediaSpeech != null && JSON.parse(oObject.loadMediaSpeech)) {
             // start cloud service
             startCloudService(false);

             var returnData = new Object();
             cloudService.fetchMediaLibrary(function (oData) {
               var mediaObject = null;
               if (oData!=null) {
                 if (oData.ReturnCode != null && oData.ReturnCode == 0) {
                   mediaObject = JSON.stringify(oData.Response.MediaLibraryTreeList);
                 }
               }
               else 
                 lInfo.logln("fetchMediaLibrary empty return");
               returnData.mediaLibrary = mediaObject;

               cloudService.fetchText2SpeechConf(function (oData) {
                 var mediaConfig = new Object();
                 if (oData!=null) {
                   if (oData.ReturnCode != null && oData.ReturnCode == 0) {
                     mediaConfig.config = oData.Response.LanguageCodeList;
                     mediaConfig.maxCalls = oData.Response.MaxTextToSpeechCalls;
                     mediaConfig.maxTextLength = oData.Response.MaxTextToSpeechLength;
                   }
                 }
                 else 
                   lInfo.logln("fetchSpeechConfiguration empty return");
                   
                 returnData.text2Speech = mediaConfig;
                 var evt = new Event("sonos-controller-playlist-stored", {value:JSON.stringify(returnData)});
                 evt.raise(); 
               });
             });
           }
           else {
             //Property.store();
             var evt = new Event("sonos-controller-playlist-stored",  {});
             evt.raise(); 
           }
         }
         if (raisedEvent.parameter.actions == 'getCurrentPresetRoom') { 
           lInfo.logln("Inside getCurrentPresetRoom");
                            
           var baseNode = Property.getNode('sonos-selected-preset');
           var roomID = -1;
           if (baseNode != null) {
             roomID = baseNode.getChild('roomID').getValue();
           }
                            
           var data = escape(JSON.stringify(objCurrentTrack));
           var evt = new Event("sonos-return-preset-room", {returnValue : roomID});
           evt.raise();
         }

    }


